@@ -93,7 +93,6 @@ class TestPath:
         stdlib_properties.discard("__class_getitem__")
         stdlib_properties.discard("__enter__")
         stdlib_properties.discard("__exit__")
-        stdlib_properties.discard("__firstlineno__")
 
         async_path = Path(path)
         anyio_properties = {
@@ -192,14 +191,9 @@ class TestPath:
         reason="Path.from_uri() is only available on Python 3.13+",
     )
     def test_from_uri(self) -> None:
-        if platform.system() == "Windows":
-            uri = "file:///C:/foo/bar"
-        else:
-            uri = "file:///foo/bar"
-
-        path = Path.from_uri(uri)
+        path = Path.from_uri("file:///foo/bar")
         assert isinstance(path, Path)
-        assert path.as_uri() == uri
+        assert path.as_uri() == "file:///foo/bar"
 
     async def test_cwd(self) -> None:
         result = await Path.cwd()
@@ -387,9 +381,6 @@ class TestPath:
         assert path.stat().st_nlink == 2
         assert target.stat().st_nlink == 2
 
-    @pytest.mark.skipif(
-        platform.system() == "Windows", reason="lchmod() does not work on Windows"
-    )
     @pytest.mark.skipif(
         not hasattr(os, "lchmod"), reason="os.lchmod() is not available"
     )
