@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import asyncio
+from typing import Any
 
 import pytest
 
@@ -12,6 +13,8 @@ from anyio import (
     Semaphore,
     WouldBlock,
     create_task_group,
+    fail_after,
+    run,
     to_thread,
     wait_all_tasks_blocked,
 )
@@ -61,6 +64,24 @@ class TestLock:
         assert not lock.locked()
         assert results == ["1", "2"]
 
+    async def test_fast_acquire(self) -> None:
+        """
+        Test that fast_acquire=True does not yield back control to the event loop when
+        there is no contention.
+
+        """
+        other_task_called = False
+
+        async def other_task() -> None:
+            nonlocal other_task_called
+            other_task_called = True
+
+        lock = Lock(fast_acquire=True)
+        async with create_task_group() as tg:
+            tg.start_soon(other_task)
+            async with lock:
+                assert not other_task_called
+
     async def test_acquire_nowait(self) -> None:
         lock = Lock()
         lock.acquire_nowait()
@@ -140,6 +161,74 @@ class TestLock:
         task1.cancel()
         await asyncio.wait_for(task2, 1)
 
+    @pytest.mark.parametrize("anyio_backend", ["asyncio"])
+    async def test_cancel_after_release(self) -> None:
+        """
+        Test that a native asyncio cancellation will not cause a lock ownership
+        to get lost between a release() and the resumption of acquire().
+
+        """
+        # Create the lock and acquire it right away so that any task acquiring it will
+        # block
+        lock = Lock()
+        lock.acquire_nowait()
+
+        # Start a task that gets blocked on trying to acquire the semaphore
+        loop = asyncio.get_running_loop()
+        task1 = loop.create_task(lock.acquire(), name="task1")
+        await asyncio.sleep(0)
+
+        # Trigger the aqcuiring task to be rescheduled, but also cancel it right away
+        lock.release()
+        task1.cancel()
+        statistics = lock.statistics()
+        assert statistics.owner
+        assert statistics.owner.name == "task1"
+        await asyncio.wait([task1], timeout=1)
+
+        # The acquire() method should've released the semaphore because acquisition
+        # failed due to cancellation
+        statistics = lock.statistics()
+        assert statistics.owner is None
+        assert statistics.tasks_waiting == 0
+        lock.acquire_nowait()
+
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_lock() -> None:
+            async with lock:
+                pass
+
+        lock = Lock()
+        statistics = lock.statistics()
+        assert not statistics.locked
+        assert statistics.owner is None
+        assert statistics.tasks_waiting == 0
+
+        run(use_lock, backend=anyio_backend_name, backend_options=anyio_backend_options)
+
+    async def test_owner_after_release(self) -> None:
+        async def taskfunc1() -> None:
+            await lock.acquire()
+            owner = lock.statistics().owner
+            assert owner
+            assert owner.name == "task1"
+            await event.wait()
+            lock.release()
+            owner = lock.statistics().owner
+            assert owner
+            assert owner.name == "task2"
+
+        event = Event()
+        lock = Lock()
+        async with create_task_group() as tg:
+            tg.start_soon(taskfunc1, name="task1")
+            await wait_all_tasks_blocked()
+            tg.start_soon(lock.acquire, name="task2")
+            await wait_all_tasks_blocked()
+            event.set()
+
 
 class TestEvent:
     async def test_event(self) -> None:
@@ -207,6 +296,21 @@ class TestEvent:
 
         assert event.statistics().tasks_waiting == 0
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_event() -> None:
+            event.set()
+            await event.wait()
+
+        event = Event()
+        assert not event.is_set()
+        assert event.statistics().tasks_waiting == 0
+
+        run(
+            use_event, backend=anyio_backend_name, backend_options=anyio_backend_options
+        )
+
 
 class TestCondition:
     async def test_contextmanager(self) -> None:
@@ -303,6 +407,22 @@ class TestCondition:
         assert not condition.statistics().lock_statistics.locked
         assert condition.statistics().tasks_waiting == 0
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_condition() -> None:
+            async with condition:
+                pass
+
+        condition = Condition()
+        assert condition.statistics().tasks_waiting == 0
+
+        run(
+            use_condition,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
+
 
 class TestSemaphore:
     async def test_contextmanager(self) -> None:
@@ -332,6 +452,24 @@ class TestSemaphore:
 
         assert semaphore.value == 2
 
+    async def test_fast_acquire(self) -> None:
+        """
+        Test that fast_acquire=True does not yield back control to the event loop when
+        there is no contention.
+
+        """
+        other_task_called = False
+
+        async def other_task() -> None:
+            nonlocal other_task_called
+            other_task_called = True
+
+        semaphore = Semaphore(1, fast_acquire=True)
+        async with create_task_group() as tg:
+            tg.start_soon(other_task)
+            async with semaphore:
+                assert not other_task_called
+
     async def test_acquire_nowait(self) -> None:
         semaphore = Semaphore(1)
         semaphore.acquire_nowait()
@@ -425,6 +563,51 @@ class TestSemaphore:
         task1.cancel()
         await asyncio.wait_for(task2, 1)
 
+    @pytest.mark.parametrize("anyio_backend", ["asyncio"])
+    async def test_cancel_after_release(self) -> None:
+        """
+        Test that a native asyncio cancellation will not cause a semaphore ownership
+        to get lost between a release() and the resumption of acquire().
+
+        """
+        # Create the semaphore in such a way that any task acquiring it will block
+        semaphore = Semaphore(0, max_value=1)
+
+        # Start a task that gets blocked on trying to acquire the semaphore
+        loop = asyncio.get_running_loop()
+        task1 = loop.create_task(semaphore.acquire())
+        await asyncio.sleep(0)
+
+        # Trigger the aqcuiring task to be rescheduled, but also cancel it right away
+        semaphore.release()
+        task1.cancel()
+        assert semaphore.value == 0
+        await asyncio.wait([task1], timeout=1)
+
+        # The acquire() method should've released the semaphore because acquisition
+        # failed due to cancellation
+        assert semaphore.value == 1
+        assert semaphore.statistics().tasks_waiting == 0
+        semaphore.acquire_nowait()
+
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_semaphore() -> None:
+            async with semaphore:
+                pass
+
+        semaphore = Semaphore(1, max_value=3)
+        assert semaphore.value == 1
+        assert semaphore.max_value == 3
+        assert semaphore.statistics().tasks_waiting == 0
+
+        run(
+            use_semaphore,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
+
 
 class TestCapacityLimiter:
     async def test_bad_init_type(self) -> None:
@@ -564,3 +747,68 @@ class TestCapacityLimiter:
             event.set()
 
         assert results == [0, 1, 2]
+
+    async def test_increase_tokens_lets_others_acquire(self) -> None:
+        limiter = CapacityLimiter(1)
+        entered_events = [Event() for _ in range(3)]
+        continue_event = Event()
+
+        async def worker(entered_event: Event) -> None:
+            async with limiter:
+                entered_event.set()
+                await continue_event.wait()
+
+        async with create_task_group() as tg:
+            for event in entered_events[:2]:
+                tg.start_soon(worker, event)
+
+            # One task should be able to acquire the limiter while the other is left
+            # waiting
+            await wait_all_tasks_blocked()
+            assert sum(ev.is_set() for ev in entered_events) == 1
+
+            # Increase the total tokens and start another worker.
+            # All tasks should be able to acquire the limiter now.
+            limiter.total_tokens = 3
+            tg.start_soon(worker, entered_events[2])
+            with fail_after(1):
+                for ev in entered_events[1:]:
+                    await ev.wait()
+
+            # Allow all tasks to exit
+            continue_event.set()
+
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_limiter() -> None:
+            async with limiter:
+                pass
+
+        limiter = CapacityLimiter(1)
+        limiter.total_tokens = 2
+
+        with pytest.raises(TypeError):
+            limiter.total_tokens = "2"  # type: ignore[assignment]
+
+        with pytest.raises(TypeError):
+            limiter.total_tokens = 3.0
+
+        assert limiter.total_tokens == 2
+        assert limiter.borrowed_tokens == 0
+        statistics = limiter.statistics()
+        assert statistics.total_tokens == 2
+        assert statistics.borrowed_tokens == 0
+        assert statistics.borrowers == ()
+        assert statistics.tasks_waiting == 0
+
+        run(
+            use_limiter,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
+
+    async def test_total_tokens_as_kwarg(self) -> None:
+        # Regression test for #515
+        limiter = CapacityLimiter(total_tokens=1)
+        assert limiter.total_tokens == 1
